from unittest import TestCase
from unittest.mock import call, Mock

import click
from box import Box
from pycarlo.core import Client

from montecarlodata.rules.rule_service import RuleService


class RuleServiceTest(TestCase):
    RULE_TABLE = """\
╒════════════════════╤═══════════════════╤══════════════════════════════════╕
│ Custom rule UUID   │ Description       │ Updated time (UTC)               │
╞════════════════════╪═══════════════════╪══════════════════════════════════╡
│ test-uuid1         │ test-description1 │ 2000-01-01 00:00:00.000000+00:00 │
├────────────────────┼───────────────────┼──────────────────────────────────┤
│ test-uuid2         │ test-description2 │ 2000-01-01 00:00:00.000000+00:00 │
╘════════════════════╧═══════════════════╧══════════════════════════════════╛"""
    LIMIT = 2
    RULE_TYPE = 'custom_sql'

    def setUp(self):
        self._client = Mock(autospec=Client)
        self._print_func = Mock(autospec=click.echo)
        self._service = RuleService(client=self._client, print_func=self._print_func)

    @staticmethod
    def _rule_response(rule_count):
        return Box(
            {
                'get_custom_rules': {
                    'edges': [
                        {
                            'node': {
                                'uuid': f'test-uuid{i}',
                                'description': f'test-description{i}',
                                'updated_time': '2000-01-01 00:00:00.000000+00:00',
                            }
                        }
                        for i in range(1, rule_count + 1)
                    ]
                }
            }
        )

    def test_list_circuit_breaker_compatible_rules(self):
        self._client.return_value = self._rule_response(self.LIMIT)

        self._service.list_rules(self.RULE_TYPE, self.LIMIT)

        self._print_func.assert_called_once_with(self.RULE_TABLE)

    def test_list_circuit_breaker_compatible_rules_with_more_available(self):
        self._client.return_value = self._rule_response(self.LIMIT + 1)

        self._service.list_rules(self.RULE_TYPE, self.LIMIT)

        expected_calls = [
            call(self.RULE_TABLE),
            call(self._service.MORE_RULES_MESSAGE),
        ]
        self._print_func.assert_has_calls(expected_calls)



from typing import Callable, Optional

import click
from pycarlo.core import Client, Query
from tabulate import tabulate

from montecarlodata.errors import manage_errors


class RuleService:
    CIRCUIT_BREAKER_RULE_HEADERS = ('Custom rule UUID', 'Description', 'Updated time (UTC)')
    MORE_RULES_MESSAGE = 'There are more monitors available. Increase the limit to view them.'

    PSEUDO_RULE_TYPE_CB_COMPATIBLE = 'circuit_breaker_compatible'
    RULE_TYPE_CUSTOM_SQL = 'custom_sql'
    RULE_TYPE_TABLE_METRIC = 'table_metric'
    RULE_TYPE_FRESHNESS = 'freshness'
    RULE_TYPES = [
        PSEUDO_RULE_TYPE_CB_COMPATIBLE,
        RULE_TYPE_CUSTOM_SQL,
        RULE_TYPE_TABLE_METRIC,
        RULE_TYPE_FRESHNESS,
    ]

    def __init__(
        self,
        client: Optional[Client] = None,
        print_func: Optional[Callable] = click.echo,
    ):
        self._client = client or Client()
        self._print_func = print_func

    @manage_errors
    def list_rules(self, rule_type: str, limit: int):
        query = Query()
        query.get_custom_rules(rule_type=rule_type, first=limit + 1).edges.node.__fields__(
            'uuid',
            'description',
            'updated_time'
        )
        response = self._client(query)

        compatible_rules = (edge.node for edge in response.get_custom_rules.edges)
        table = [(rule.uuid, rule.description, rule.updated_time) for rule in compatible_rules]

        more_rules_available = False
        if len(table) > limit:
            table = table[:-1]
            more_rules_available = True

        self._print_func(tabulate(table, headers=self.CIRCUIT_BREAKER_RULE_HEADERS, tablefmt='fancy_grid'))

        if more_rules_available:
            self._print_func(self.MORE_RULES_MESSAGE)



from unittest import TestCase
from unittest.mock import Mock

from montecarlodata.common.user import UserService
from montecarlodata.integrations.info.status import OnboardingStatusService
from tests.helpers import capture_function
from tests.test_common_user import _SAMPLE_CONFIG


class OnboardingStatusTest(TestCase):
    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)

        self._service = OnboardingStatusService(_SAMPLE_CONFIG, user_service=self._user_service_mock)

    def test_display_integrations_with_no_connections(self):
        self._user_service_mock.warehouses = None
        self._user_service_mock.bi_containers = None

        std_out = capture_function(self._service.display_integrations, {'table_format': 'plain'}).std_out
        self.assertEqual(std_out.getvalue().strip(), 'Integration    ID    Connection    Created on (UTC)')

    def test_display_integrations_with_connections(self):
        sample_uuid, sample_type1, sample_type2, sample_time = '1', 'foo', 'bar', 'now'
        sample_connection_identifier1 = {
            'key': 'host',
            'value': 'foo'
        }
        sample_connection_identifier2 = {
            'key': 'host',
            'value': 'bar'
        }
        self._user_service_mock.warehouses = [
            {'connections': [{'uuid': sample_uuid, 'type': sample_type1, 'connectionIdentifier': sample_connection_identifier1, 'createdOn': sample_time}]}]
        self._user_service_mock.bi_containers = [
            {'connections': [{'uuid': sample_uuid, 'type': sample_type2, 'connectionIdentifier': sample_connection_identifier2, 'createdOn': sample_time}]}]

        expected_output = 'Integration      ID  Connection    Created on (UTC)\nfoo               1  host: foo     now\nbar               1  host: bar     now'
        std_out = capture_function(self._service.display_integrations, {'table_format': 'plain'}).std_out
        self.assertEqual(std_out.getvalue().strip(), expected_output)



import base64
from unittest import TestCase
from unittest.mock import Mock, patch

from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.warehouse.warehouses import WarehouseOnboardingService
from montecarlodata.queries.onboarding import TEST_DATABASE_CRED_MUTATION, TEST_SNOWFLAKE_CRED_MUTATION, \
    TEST_BQ_CRED_MUTATION
from montecarlodata.utils import GqlWrapper, AwsClientWrapper
from tests.test_base_onboarding import _SAMPLE_BASE_OPTIONS
from tests.test_common_user import _SAMPLE_CONFIG


class WarehouseOnBoardingTest(TestCase):
    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = WarehouseOnboardingService(
            _SAMPLE_CONFIG,
            request_wrapper=self._request_wrapper_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock
        )

    @patch.object(WarehouseOnboardingService, 'onboard')
    def test_redshift_flow(self, onboard_mock):
        expected_options = {**{'connectionType': 'redshift', 'warehouseType': 'redshift'}, **_SAMPLE_BASE_OPTIONS}

        self._service.onboard_redshift(**_SAMPLE_BASE_OPTIONS)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_DATABASE_CRED_MUTATION,
            validation_response='testDatabaseCredentials',
            connection_type='redshift',
            **expected_options
        )

    @patch.object(WarehouseOnboardingService, 'onboard')
    def test_snowflake_flow(self, onboard_mock):
        expected_options = {**{'warehouseType': 'snowflake'}, **_SAMPLE_BASE_OPTIONS}

        self._service.onboard_snowflake(**_SAMPLE_BASE_OPTIONS)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_SNOWFLAKE_CRED_MUTATION,
            validation_response='testSnowflakeCredentials',
            connection_type='snowflake',
            **expected_options
        )

    @patch.object(WarehouseOnboardingService, 'onboard')
    @patch('montecarlodata.integrations.onboarding.warehouse.warehouses.read_as_base64')
    def test_bq_flow(self, read_as_base64_mock, onboard_mock):
        file_path, service_json = 'foo', 'bar'
        base64_service_json = base64.b64encode(service_json.encode('utf-8'))

        input_options = {'ServiceFile': file_path, **_SAMPLE_BASE_OPTIONS}
        expected_options = {
            **{'warehouseType': 'bigquery', 'serviceJson': base64_service_json.decode()}, **_SAMPLE_BASE_OPTIONS}

        read_as_base64_mock.return_value = base64_service_json

        self._service.onboard_bq(**input_options)
        read_as_base64_mock.assert_called_once_with(file_path)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_BQ_CRED_MUTATION,
            validation_response='testBqCredentials',
            connection_type='bigquery',
            **expected_options
        )



import json
import os
import pathlib
from unittest import TestCase
from unittest.mock import Mock, patch

from montecarlodata.common.data import MonolithResponse
from montecarlodata.iac.mc_config_service import MonteCarloConfigService
from montecarlodata.queries.iac import CREATE_OR_UPDATE_MONTE_CARLO_CONFIG_TEMPLATE
from montecarlodata.utils import GqlWrapper
from tests.test_common_user import _SAMPLE_CONFIG


class MonteCarloConfigServiceTest(TestCase):
    def setUp(self) -> None:
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)

    @patch('os.getcwd')
    def test_standalone_configs(self, getcwd):
        project_dir = self._get_project_dir('standalone_configs')
        getcwd.return_value = project_dir
        service = MonteCarloConfigService(_SAMPLE_CONFIG, self._request_wrapper_mock)
        files, template, _ = service.compile()
        self.assertEqual(len(files), 2)
        self.assertEqual(
            template,
            {
                'field_health': [{
                    'table': 'analytics:prod.client_hub',
                    'fields': ['account_id']
                }],
                'dimension_tracking': [{
                    'table': 'analytics:prod.customer_360',
                    'field': 'account_id'
                }]
            }
        )

    @patch('os.getcwd')
    def test_embedded_dbt_configs(self, getcwd):
        getcwd.return_value = self._get_project_dir('embedded_dbt_configs')
        service = MonteCarloConfigService(_SAMPLE_CONFIG, self._request_wrapper_mock)
        files, template, _ = service.compile()

        self.assertEqual(len(files), 4)
        self.assertEqual(
            template,
            {
                'field_health': [{
                    'table': 'analytics:prod_lineage.lineage_nodes',
                    'timestamp_field': 'created'
                }, {
                    'table': 'analytics:prod.abc'
                }, {
                    'table': 'analytics:prod.client_hub',
                    'fields': ['account_id']
                }],
                'freshness': [{
                    'table': 'analytics:prod.abc',
                    'freshness_threshold': 30,
                    'schedule': {
                        'type': 'fixed',
                        'interval_minutes': 30,
                        'start_time': '2021-07-27T19:51:00'
                    }
                }],
                'dimension_tracking': [{
                    'table': 'analytics:prod.customer_360',
                    'field': 'account_id'
                }]
            }
        )

    @patch('os.getcwd')
    def test_invalid_configs(self, getcwd):
        project_dir = self._get_project_dir('invalid_configs')
        getcwd.return_value = project_dir
        service = MonteCarloConfigService(_SAMPLE_CONFIG, self._request_wrapper_mock)
        service._abort_on_error = False

        files, template, errors_by_file = service.compile(abort_on_error=False)
        errors = sorted(list(errors_by_file.items()), key=lambda x: x[0])

        file, error = errors[0]
        self.assertTrue(file.endswith('dir1/dir2/monitors.yml'))
        self.assertEqual(error, ['"custom_sql" property should be a list.'])
        file, error = errors[1]
        self.assertTrue(file.endswith('dir1/monitors.yml'))
        self.assertEqual(error, ['"field_health" property should be a list.'])

    @patch('os.getcwd')
    def test_apply(self, getcwd):
        project_dir = self._get_project_dir('standalone_configs')
        getcwd.return_value = project_dir
        namespace = 'foo'
        service = MonteCarloConfigService(_SAMPLE_CONFIG, self._request_wrapper_mock)

        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(
            data={
                'response': {
                    'resourceModifications': [
                        {
                            'type': 'ResourceModificationType.UPDATE',
                            'description': 'Monitor: type=stats, table=analytics:prod.client_hub',
                            'resourceAsJson': '{"uuid": "ed4d07c3-58fd-44d0-8b2d-c1b020f45a69", "resource": null, "name": "monitor|type=stats|table=analytics:prod.client_hub|timestamp_field=<<NULL>>|where_condition=<<NULL>>", "table": "analytics:prod.customer_360", "type": "stats", "fields": [], "timestamp_field": null, "where_condition": null}'
                        }, {
                            'type': 'ResourceModificationType.UPDATE',
                            'description': 'Monitor: type=categories, table=analytics:prod.customer_360',
                            'resourceAsJson': '{"uuid": "ec3b0a80-d088-4dbe-acf5-150caf041574", "resource": null, "name": "monitor|type=categories|table=analytics:prod.customer_360|timestamp_field=<<NULL>>|where_condition=<<NULL>>|fields=account_id", "table": "analytics:prod.customer_360", "type": "categories", "fields": ["account_id"], "timestamp_field": null, "where_condition": null}'
                        }
                    ],
                    'changesApplied': True,
                    'errorsAsJson': '{}'
                }
            }
        )

        response = service.apply(namespace)

        config_template_as_dict = {
            'field_health': [{
                'table': 'analytics:prod.client_hub',
                'fields': ['account_id']
            }],
            'dimension_tracking': [{
                'table': 'analytics:prod.customer_360',
                'field': 'account_id'
            }]
        }

        self._request_wrapper_mock.make_request_v2.assert_called_once_with(
            query=CREATE_OR_UPDATE_MONTE_CARLO_CONFIG_TEMPLATE,
            operation='createOrUpdateMonteCarloConfigTemplate',
            variables=dict(
                namespace=namespace,
                configTemplateJson=json.dumps(config_template_as_dict),
                dryRun=False,
                resource=None
            )
        )

        self.assertEqual(response.errors, {})
        self.assertEqual(len(response.resource_modifications), 2)

    @patch('os.getcwd')
    def test_apply_with_errors(self, getcwd):
        project_dir = self._get_project_dir('standalone_configs')
        getcwd.return_value = project_dir
        namespace = 'foo'
        service = MonteCarloConfigService(_SAMPLE_CONFIG, self._request_wrapper_mock)

        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(
            data={
                'response': {
                    'resourceModifications': [],
                    'changesApplied': False,
                    'errorsAsJson': '{"validation_errors": {"monitors": {"0": {"type": ["Unknown field."]}}}}'
                }
            })

        response = service.apply(namespace, abort_on_error=False)

        config_template_as_dict = {
            'field_health': [{
                'table': 'analytics:prod.client_hub',
                'fields': ['account_id']
            }],
            'dimension_tracking': [{
                'table': 'analytics:prod.customer_360',
                'field': 'account_id'
            }]
        }

        self._request_wrapper_mock.make_request_v2.assert_called_once_with(
            query=CREATE_OR_UPDATE_MONTE_CARLO_CONFIG_TEMPLATE,
            operation='createOrUpdateMonteCarloConfigTemplate',
            variables=dict(
                namespace=namespace,
                configTemplateJson=json.dumps(config_template_as_dict),
                dryRun=False,
                resource=None
            )
        )

        self.assertEqual(response.errors, {'validation_errors': {'monitors': {'0': {'type': ['Unknown field.']}}}})
        self.assertEqual(len(response.resource_modifications), 0)

    def _get_project_dir(self, dir_name: str):
        return os.path.join(pathlib.Path(__file__).parent.resolve(), 'test_resources', dir_name)
